@@ -130,7 +130,7 @@ class HIGAlertDialog(gtk.Dialog):
 
 class BaseDialog(gtk.Dialog):
     def __init__(self, parent=None, title='', flags=0, buttons=()):
-        if parent and not isinstance(parent, gtk.Window):
+        if parent and not isinstance(parent, (gtk.Window, gtk.Dialog)):
             raise TypeError("parent needs to be None or a gtk.Window subclass")
 
         if not flags and parent:
@@ -147,16 +147,16 @@ def messagedialog(dialog_type, short, long=None, parent=None,
                   buttons=gtk.BUTTONS_OK, default=-1):
     """Create and show a MessageDialog.
 
-    @param dialog_type: one of constants
+    :param dialog_type: one of constants
       - gtk.MESSAGE_INFO
       - gtk.MESSAGE_WARNING
       - gtk.MESSAGE_QUESTION
       - gtk.MESSAGE_ERROR
-    @param short:       A header text to be inserted in the dialog.
-    @param long:        A long description of message.
-    @param parent:      The parent widget of this dialog
-    @type parent:       a gtk.Window subclass
-    @param buttons:     The button type that the dialog will be display,
+    :param short:       A header text to be inserted in the dialog.
+    :param long:        A long description of message.
+    :param parent:      The parent widget of this dialog
+    :type parent:       a gtk.Window subclass
+    :param buttons:     The button type that the dialog will be display,
       one of the constants:
        - gtk.BUTTONS_NONE
        - gtk.BUTTONS_OK
@@ -166,7 +166,7 @@ def messagedialog(dialog_type, short, long=None, parent=None,
        - gtk.BUTTONS_OK_CANCEL
       or a tuple or 2-sized tuples representing label and response. If label
       is a stock-id a stock icon will be displayed.
-    @param default: optional default response id
+    :param default: optional default response id
     """
     if buttons in (gtk.BUTTONS_NONE, gtk.BUTTONS_OK, gtk.BUTTONS_CLOSE,
                    gtk.BUTTONS_CANCEL, gtk.BUTTONS_YES_NO,
@@ -179,7 +179,7 @@ def messagedialog(dialog_type, short, long=None, parent=None,
                 "buttons must be a GtkButtonsTypes constant or a tuple")
         dialog_buttons = gtk.BUTTONS_NONE
 
-    if parent and not isinstance(parent, gtk.Window):
+    if parent and not isinstance(parent, (gtk.Window, gtk.Dialog)):
         raise TypeError("parent must be a gtk.Window subclass")
 
     d = HIGAlertDialog(parent=parent, flags=gtk.DIALOG_MODAL,
@@ -238,11 +238,11 @@ def yesno(text, parent=None, default=gtk.RESPONSE_YES,
 def open(title='', parent=None, patterns=None, folder=None, filter=None,
          with_file_chooser=False):
     """Displays an open dialog.
-    @param title: the title of the folder, defaults to 'Select folder'
-    @param parent: parent gtk.Window or None
-    @param patterns: a list of pattern strings ['*.py', '*.pl'] or None
-    @param folder: initial folder or None
-    @param filter: a filter to use or None, is incompatible with patterns
+    :param title: the title of the folder, defaults to 'Select folder'
+    :param parent: parent gtk.Window or None
+    :param patterns: a list of pattern strings ['*.py', '*.pl'] or None
+    :param folder: initial folder or None
+    :param filter: a filter to use or None, is incompatible with patterns
     """
 
     ffilter = filter
@@ -296,9 +296,9 @@ def open(title='', parent=None, patterns=None, folder=None, filter=None,
 
 def selectfolder(title='', parent=None, folder=None):
     """Displays a select folder dialog.
-    @param title: the title of the folder, defaults to 'Select folder'
-    @param parent: parent gtk.Window or None
-    @param folder: initial folder or None
+    :param title: the title of the folder, defaults to 'Select folder'
+    :param parent: parent gtk.Window or None
+    :param folder: initial folder or None
     """
 
     filechooser = gtk.FileChooserDialog(
@@ -377,11 +377,11 @@ def save(title='', parent=None, current_name='', folder=None):
 def password(primary='', secondary='', parent=None):
     """
     Shows a password dialog and returns the password entered in the dialog
-    @param primary: primary text
-    @param secondary: secondary text
-    @param parent: a gtk.Window subclass or None
-    @returns: the password or None if none specified
-    @rtype: string or None
+    :param primary: primary text
+    :param secondary: secondary text
+    :param parent: a gtk.Window subclass or None
+    :returns: the password or None if none specified
+    :rtype: string or None
     """
     if not primary:
         raise ValueError("primary cannot be empty")
