@@ -42,7 +42,8 @@ from kiwi.enums import SearchFilterPosition
 from kiwi.interfaces import ISearchFilter
 from kiwi.python import enum
 from kiwi.ui.delegates import SlaveDelegate
-from kiwi.ui.objectlist import ObjectList, SummaryLabel, SearchColumn
+from kiwi.ui.objectlist import (ObjectList, ObjectTree,
+                                SummaryLabel, SearchColumn)
 from kiwi.ui.widgets.combo import ProxyComboBox
 from kiwi.ui.widgets.entry import ProxyDateEntry
 from kiwi.utils import gsignal
@@ -58,7 +59,7 @@ class DateSearchOption(object):
     """
     Base class for Date search options
     A date search option is an interval of dates
-    @cvar name: name of the search option
+    :cvar name: name of the search option
     """
     name = None
 
@@ -68,8 +69,8 @@ class DateSearchOption(object):
     def get_interval(self):
         """
         Get start and end date.
-        @returns: start date, end date
-        @rtype: datetime.date tuple
+        :returns: start date, end date
+        :rtype: datetime.date tuple
         """
 
 class Any(DateSearchOption):
@@ -147,8 +148,8 @@ class NumberSearchOption(object):
     """
     Base class for Number search options
     A number search option is an interval of numbers
-    @cvar name: name of the search option
-    @cvar numbers: how many numbers must the user input: 0, 1 or 2
+    :cvar name: name of the search option
+    :cvar numbers: how many numbers must the user input: 0, 1 or 2
     """
     name = None
     numbers = 0
@@ -156,7 +157,7 @@ class NumberSearchOption(object):
     def get_interval(self, start, end):
         """
         Get start and end interval.
-        @returns: start, end
+        :returns: start, end
         """
 
 class Between(NumberSearchOption):
@@ -217,15 +218,15 @@ class SearchFilter(gtk.HBox):
     """
     A base classed used by common search filters
     """
-    label = gobject.property(type=str, flags=(gobject.PARAM_READWRITE |
-                                              gobject.PARAM_CONSTRUCT_ONLY))
+    label = gobject.property(type=str, flags=(gobject.PARAM_READWRITE))
     gsignal('changed')
     gsignal('removed')
 
     implements(ISearchFilter)
 
     def __init__(self, label=''):
-        self.__gobject_init__(label=label)
+        gtk.HBox.__init__(self)
+        self.props.label = label
         self._label = label
         self._remove_button = None
 
@@ -269,7 +270,7 @@ class SearchFilter(gtk.HBox):
 
     def get_description(self):
         """Returns a description of the search filter.
-        @returns: a string describing the search filter.
+        :returns: a string describing the search filter.
         """
         raise NotImplementedError
 
@@ -291,7 +292,7 @@ class DateSearchFilter(SearchFilter):
     def __init__(self, label=''):
         """
         Create a new DateSearchFilter object.
-        @param label: name of the search filter
+        :param label: name of the search filter
         """
         self._options = {}
         SearchFilter.__init__(self, label=label)
@@ -383,8 +384,8 @@ class DateSearchFilter(SearchFilter):
     def add_option(self, option_type, position=-2):
         """
         Adds a date option
-        @param option_type: option to add
-        @type option_type: a L{DateSearchOption} subclass
+        :param option_type: option to add
+        :type option_type: a :class:`DateSearchOption` subclass
         """
         option = option_type()
         num = len(self.mode) + position
@@ -395,9 +396,9 @@ class DateSearchFilter(SearchFilter):
         """
         Adds a fixed option, eg one for which date is not
         possible to modify.
-        @param name: name of the option
-        @param date: fixed data
-        @param position: position to add the option at
+        :param name: name of the option
+        :param date: fixed data
+        :param position: position to add the option at
         """
         option_type = type('', (FixedDateSearchOption,),
                            dict(name=name, date=date))
@@ -408,10 +409,10 @@ class DateSearchFilter(SearchFilter):
         """
         Adds a fixed option interval, eg one for which the dates are not
         possible to modify.
-        @param name: name of the option
-        @param start: start of the fixed interval
-        @param end: end of the fixed interval
-        @param position: position to add the option at
+        :param name: name of the option
+        :param start: start of the fixed interval
+        :param end: end of the fixed interval
+        :param position: position to add the option at
         """
         option_type = type('', (FixedIntervalSearchOption,),
                            dict(name=name, start=start, end=end))
@@ -432,23 +433,23 @@ class DateSearchFilter(SearchFilter):
     def get_start_date(self):
         """
         Get the start date.
-        @returns: start date
-        @rtype: datetime.date or None
+        :returns: start date
+        :rtype: datetime.date or None
         """
         return self.start_date.get_date()
 
     def get_end_date(self):
         """
         Get the end date.
-        @returns: end date
-        @rtype: datetime.date or None
+        :returns: end date
+        :rtype: datetime.date or None
         """
         return self.end_date.get_date()
 
     def set_use_date_entries(self, use_date_entries):
         """
         Toggles the visibility of the user selectable date entries
-        @param use_date_entries:
+        :param use_date_entries:
         """
         self.from_label.props.visible = use_date_entries
         self.to_label.props.visible = use_date_entries
@@ -461,8 +462,8 @@ class DateSearchFilter(SearchFilter):
         Data or position can be sent in. If nothing
         is sent in the first item will be selected, if any
 
-        @param data: data to select
-        @param position: position of data to select
+        :param data: data to select
+        :param position: position of data to select
         """
         if data is not None and position is not None:
             raise TypeError("You can't send in both data and position")
@@ -597,10 +598,11 @@ class ComboSearchFilter(SearchFilter):
     def __init__(self, label='', values=None):
         """
         Create a new ComboSearchFilter object.
-        @param name: name of the search filter
-        @param values: items to put in the combo, see
-            L{kiwi.ui.widgets.combo.ProxyComboBox.prefill}
+        :param name: name of the search filter
+        :param values: items to put in the combo, see
+            :class:`kiwi.ui.widgets.combo.ProxyComboBox.prefill`
         """
+        self._block_updates = False
         SearchFilter.__init__(self, label=label)
         label = gtk.Label(label)
         self.pack_start(label, False, False)
@@ -609,7 +611,7 @@ class ComboSearchFilter(SearchFilter):
 
         self.combo = ProxyComboBox()
         if values:
-            self.combo.prefill(values)
+            self.update_values(values)
         self.combo.connect('content-changed', self._on_combo__content_changed)
         self.pack_start(self.combo, False, False, 6)
         self.combo.show()
@@ -624,7 +626,6 @@ class ComboSearchFilter(SearchFilter):
                                  value=value)
         if hasattr(value, 'id'):
             state.value_id = value.id
-            state.value = None
         return state
 
     def set_state(self, value, value_id=None):
@@ -637,6 +638,11 @@ class ComboSearchFilter(SearchFilter):
                     break
         self.select(value)
 
+    def update_values(self, values):
+        self._block_updates = True
+        self.combo.prefill(values)
+        self._block_updates = False
+
     def get_title_label(self):
         return self.title_label
 
@@ -657,7 +663,7 @@ class ComboSearchFilter(SearchFilter):
     def select(self, data):
         """
         selects an item in the combo
-        @param data: what to select
+        :param data: what to select
         """
         self.combo.select(data)
 
@@ -666,7 +672,8 @@ class ComboSearchFilter(SearchFilter):
     #
 
     def _on_combo__content_changed(self, mode):
-        self.emit('changed')
+        if not self._block_updates:
+            self.emit('changed')
 
 
 # Ported from evolution
@@ -729,15 +736,16 @@ class StringSearchFilter(SearchFilter):
     """
     - a label
     - an entry
-    @ivar entry: the entry
-    @ivar label: the label
+    :ivar entry: the entry
+    :ivar label: the label
     """
-    def __init__(self, label, chars=0):
+    def __init__(self, label, chars=0, container=None):
         """
         Create a new StringSearchFilter object.
-        @param label: label of the search filter
-        @param chars: maximum number of chars used by the search entry
+        :param label: label of the search filter
+        :param chars: maximum number of chars used by the search entry
         """
+        self._container = container
         SearchFilter.__init__(self, label=label)
         self.title_label = gtk.Label(label)
         self.pack_start(self.title_label, False, False)
@@ -772,7 +780,7 @@ class StringSearchFilter(SearchFilter):
 
     def _add_option(self, option_type, position=-2):
         option = option_type()
-        num = len(self.mode) + position
+        num = abs(position)
         self.mode.insert_item(num, option.name, option_type)
         self._options[option_type] = option
 
@@ -789,11 +797,22 @@ class StringSearchFilter(SearchFilter):
     def _on_entry__changed(self, entry):
         entry.props.secondary_icon_sensitive = bool(entry.get_text())
 
+    def _position_filter_menu(self, data):
+        window = self.entry.get_icon_window(gtk.ENTRY_ICON_PRIMARY)
+        x, y = window.get_origin()
+        border = self.entry.style_get_property('progress-border')
+        return (x, y + window.get_size()[1] + border.bottom, True)
+
     def _on_entry__icon_release(self, entry, icon_pos, event):
         if icon_pos == gtk.ENTRY_ICON_SECONDARY:
             entry.set_text("")
             entry.grab_focus()
             self.emit('changed')
+        elif icon_pos == gtk.ENTRY_ICON_PRIMARY:
+            if not self._container:
+                return
+            self._container.menu.popup(None, None,
+                                       self._position_filter_menu, 0, event.time)
 
     #
     # SearchFilter
@@ -803,7 +822,7 @@ class StringSearchFilter(SearchFilter):
         option = self.mode.get_selected_data()
         return StringQueryState(filter=self,
                                 text=self.entry.get_text(),
-                                mode=option.mode)
+                                mode=option and option.mode)
 
     def set_state(self, text, mode=None):
         self.entry.set_text(text)
@@ -842,7 +861,7 @@ class NumberSearchFilter(SearchFilter):
     def __init__(self, label=''):
         """
         Create a new NumberSearchFilter object.
-        @param label: name of the search filter
+        :param label: name of the search filter
         """
 
         self._options = {}
@@ -884,7 +903,7 @@ class NumberSearchFilter(SearchFilter):
     def set_digits(self, digits):
         """
         Number of decimal place to be displayed
-        @param digits: number of decimal places
+        :param digits: number of decimal places
         """
         self.start.set_digits(digits)
         self.end.set_digits(digits)
@@ -966,8 +985,8 @@ class NumberSearchFilter(SearchFilter):
     def add_option(self, option_type, position=-2):
         """
         Adds a date option
-        @param option_type: option to add
-        @type option_type: a L{NumberSearchOption} subclass
+        :param option_type: option to add
+        :type option_type: a :class:`NumberSearchOption` subclass
         """
         option = option_type()
         num = len(self.mode) + position
@@ -1002,30 +1021,55 @@ class SearchResults(ObjectList):
     def __init__(self, columns):
         ObjectList.__init__(self, columns)
 
+    def add_results(self, results):
+        self.extend(results)
+
+
+class SearchResultsTree(ObjectTree):
+    def __init__(self, columns):
+        ObjectTree.__init__(self, columns)
+
+    def add_results(self, results):
+        for result in results:
+            self._add_result(result)
+
+    def _add_result(self, result):
+        parent = result.get_parent()
+        if parent:
+            self._add_result(parent)
+        if not result in self:
+            self.append(parent, result)
+            if parent:
+                self.expand(parent)
+
 
 class SearchContainer(gtk.VBox):
     """
     A search container is a widget which consists of:
-    - search entry (w/ a label) (L{StringSearchFilter})
+    - search entry (w/ a label) (:class:`StringSearchFilter`)
     - search button
-    - objectlist result (L{SearchResult})
-    - a query executer (L{kiwi.db.query.QueryExecuter})
+    - objectlist result (:class:`SearchResults` or class:`SearchResultsTree)
+    - a query executer (:class:`kiwi.db.query.QueryExecuter`)
 
     Additionally you can add a number of search filters to the SearchContainer.
     You can chose if you want to add the filter in the top-left corner
-    of bottom, see L{SearchFilterPosition}
+    of bottom, see :class:`SearchFilterPosition`
     """
     __gtype_name__ = 'SearchContainer'
     filter_label = gobject.property(type=str)
     results_class = SearchResults
     gsignal("search-completed", object, object)
 
-    def __init__(self, columns=None, chars=25):
+    def __init__(self, columns=None, tree=False, chars=25):
         """
         Create a new SearchContainer object.
-        @param columns: a list of L{kiwi.ui.objectlist.Column}
-        @param chars: maximum number of chars used by the search entry
+        :param columns: a list of :class:`kiwi.ui.objectlist.Column`
+        :param tree: if we should list the results as a tree
+        :param chars: maximum number of chars used by the search entry
         """
+        if tree:
+            self.results_class = SearchResultsTree
+
         gtk.VBox.__init__(self)
         self._columns = columns
         self._search_filters = []
@@ -1033,7 +1077,8 @@ class SearchContainer(gtk.VBox):
         self._auto_search = True
         self._summary_label = None
 
-        search_filter = StringSearchFilter(_('Search:'), chars=chars)
+        search_filter = StringSearchFilter(_('Search:'), chars=chars,
+                                           container=self)
         search_filter.connect('changed', self._on_search_filter__changed)
         self._search_filters.append(search_filter)
         self._primary_filter = search_filter
@@ -1088,10 +1133,10 @@ class SearchContainer(gtk.VBox):
                    columns=None, callback=None):
         """
         Adds a search filter
-        @param search_filter: the search filter
-        @param postition: a L{SearchFilterPosition} enum
-        @param columns:
-        @param callback:
+        :param search_filter: the search filter
+        :param postition: a :class:`SearchFilterPosition` enum
+        :param columns:
+        :param callback:
         """
 
         if not isinstance(search_filter, SearchFilter):
@@ -1115,7 +1160,7 @@ class SearchContainer(gtk.VBox):
                     "You need to set an executor before calling set_filters "
                     "with columns or callback set")
 
-        assert not search_filter.parent
+        assert not search_filter.get_parent()
         self.set_filter_position(search_filter, position)
         search_filter.connect('changed', self._on_search_filter__changed)
         search_filter.connect('removed', self._on_search_filter__remove)
@@ -1140,11 +1185,11 @@ class SearchContainer(gtk.VBox):
     def set_filter_position(self, search_filter, position):
         """
         Set the the filter position.
-        @param search_filter:
-        @param position:
+        :param search_filter:
+        :param position:
         """
-        if search_filter.parent:
-            search_filter.parent.remove(search_filter)
+        if search_filter.get_parent():
+            search_filter.get_parent().remove(search_filter)
 
         if position == SearchFilterPosition.TOP:
             self.hbox.pack_start(search_filter, False, False)
@@ -1156,11 +1201,11 @@ class SearchContainer(gtk.VBox):
     def get_filter_position(self, search_filter):
         """
         Get filter by position.
-        @param search_filter:
+        :param search_filter:
         """
-        if search_filter.parent == self.hbox:
+        if search_filter.get_parent() == self.hbox:
             return SearchFilterPosition.TOP
-        elif search_filter.parent == self:
+        elif search_filter.get_parent() == self:
             return SearchFilterPosition.BOTTOM
         else:
             raise AssertionError(search_filter)
@@ -1168,8 +1213,8 @@ class SearchContainer(gtk.VBox):
     def set_query_executer(self, querty_executer):
         """
         Ties a QueryExecuter instance to the SearchContainer class
-        @param querty_executer: a querty executer
-        @type querty_executer: a L{QueryExecuter} subclass
+        :param querty_executer: a querty executer
+        :type querty_executer: a :class:`QueryExecuter` subclass
         """
         if not isinstance(querty_executer, QueryExecuter):
             raise TypeError("querty_executer must be a QueryExecuter instance")
@@ -1179,8 +1224,8 @@ class SearchContainer(gtk.VBox):
     def get_query_executer(self):
         """
         Fetchs the QueryExecuter for the SearchContainer
-        @returns: a querty executer
-        @rtype: a L{QueryExecuter} subclass
+        :returns: a querty executer
+        :rtype: a :class:`QueryExecuter` subclass
         """
         return self._query_executer
 
@@ -1189,7 +1234,7 @@ class SearchContainer(gtk.VBox):
         Fetches the primary filter for the SearchContainer.
         The primary filter is the filter attached to the standard entry
         normally used to do free text searching
-        @returns: the primary filter
+        :returns: the primary filter
         """
         return self._primary_filter
 
@@ -1212,7 +1257,7 @@ class SearchContainer(gtk.VBox):
         """
         Enables/Disables auto search which means that the search result box
         is automatically populated when a filter changes
-        @param auto_search: True to enable, False to disable
+        :param auto_search: True to enable, False to disable
         """
         self._auto_search = auto_search
 
@@ -1238,10 +1283,10 @@ class SearchContainer(gtk.VBox):
                           parent=None):
         """
         Adds a summary label to the result set
-        @param column: the column to sum from
-        @param label: the label to use, defaults to 'Total:'
-        @param format: the format, defaults to '%%s', must include '%%s'
-        @param parent: the parent widget a label should be added to or
+        :param column: the column to sum from
+        :param label: the label to use, defaults to 'Total:'
+        :param format: the format, defaults to '%%s', must include '%%s'
+        :param parent: the parent widget a label should be added to or
            None if it should be added to the SearchContainer
         """
         if not '%s' in format:
@@ -1259,7 +1304,7 @@ class SearchContainer(gtk.VBox):
                 parent))
 
         if self._summary_label:
-            self._summary_label.parent.remove(self._summary_label)
+            self._summary_label.get_parent().remove(self._summary_label)
         self._summary_label = SummaryLabel(klist=self.results,
                                            column=column,
                                            label=label,
@@ -1272,7 +1317,7 @@ class SearchContainer(gtk.VBox):
 
     def add_results(self, results):
         self.results.clear()
-        self.results.extend(results)
+        self.results.add_results(results)
 
     def get_filter_states(self):
         dict_state = {}
@@ -1288,6 +1333,7 @@ class SearchContainer(gtk.VBox):
                 data['value'] = state.value
                 if hasattr(state, 'value_id'):
                     data['value_id'] = state.value_id
+                    data['value'] = None
             elif isinstance(state, NumberIntervalQueryState):
                 data['start'] = state.start
                 data['end'] = state.end
@@ -1364,13 +1410,6 @@ class SearchContainer(gtk.VBox):
         self.label_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         self.combo_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
 
-        add = SearchFilterButton(label=_('Filter'), stock=gtk.STOCK_ADD)
-        add.connect('clicked', self._on_add_field_clicked)
-        add.show()
-        self.hbox.pack_end(add, False, False, 0)
-
-        self.add_filter_button = add
-
         self.menu = gtk.Menu()
         for column in self._columns:
             if not isinstance(column, SearchColumn):
@@ -1388,18 +1427,6 @@ class SearchContainer(gtk.VBox):
             menu_item.connect('activate', self._on_menu_item_activate)
             self.menu.append(menu_item)
 
-        if not len(self.menu):
-            self.add_filter_button.hide()
-
-    def _position_filter_menu(self, data):
-        alloc = self.add_filter_button.get_allocation()
-        x, y = self.add_filter_button.window.get_origin()
-
-        return (x + alloc.x, y + alloc.y + alloc.height, True)
-
-    def _on_add_field_clicked(self, button):
-        self.menu.popup(None, None, self._position_filter_menu, 0, 0L)
-
     def _on_menu_item_activate(self, item):
         column = item.get_data('column')
 
@@ -1453,18 +1480,17 @@ SearchContainer.install_child_property(
 
 
 class SearchSlaveDelegate(SlaveDelegate):
-    def __init__(self, columns):
+    def __init__(self, columns, tree=False):
         """
         Create a new SearchSlaveDelegate object.
-        @ivar results: the results list of the container
-        @ivar search: the L{SearchContainer}
+        :ivar results: the results list of the container
+        :ivar search: the :class:`SearchContainer`
         """
-        self.search = SearchContainer(columns)
+        self.search = SearchContainer(columns, tree=tree)
         SlaveDelegate.__init__(self, toplevel=self.search)
         self.results = self.search.results
         self.search.show()
 
-
     #
     # Public API
     #
@@ -1472,26 +1498,26 @@ class SearchSlaveDelegate(SlaveDelegate):
     def add_filter(self, search_filter, position=SearchFilterPosition.BOTTOM,
                    columns=None, callback=None):
         """
-        See L{SearchSlaveDelegate.add_filter}
+        See :class:`SearchSlaveDelegate.add_filter`
         """
         self.search.add_filter(search_filter, position, columns, callback)
 
     def set_query_executer(self, querty_executer):
         """
-        See L{SearchSlaveDelegate.set_query_executer}
+        See :class:`SearchSlaveDelegate.set_query_executer`
         """
         self.search.set_query_executer(querty_executer)
 
     def set_text_field_columns(self, columns):
         """
-        See L{SearchSlaveDelegate.set_text_field_columns}
+        See :class:`SearchSlaveDelegate.set_text_field_columns`
         """
         self.search.set_text_field_columns(columns)
 
     def get_primary_filter(self):
         """
         Fetches the primary filter of the SearchSlaveDelegate
-        @returns: primary filter
+        :returns: primary filter
         """
         return self.search.get_primary_filter()
 
@@ -1522,13 +1548,13 @@ class SearchSlaveDelegate(SlaveDelegate):
     def set_summary_label(self, column, label='Total:', format='%s',
                           parent=None):
         """
-        See L{SearchContainer.set_summary_label}
+        See :class:`SearchContainer.set_summary_label`
         """
         self.search.set_summary_label(column, label, format, parent)
 
     def enable_advanced_search(self):
         """
-        See L{SearchContainer.enable_advanced_search}
+        See :class:`SearchContainer.enable_advanced_search`
         """
         self.search.enable_advanced_search()
 
@@ -1542,7 +1568,7 @@ class SearchSlaveDelegate(SlaveDelegate):
     def get_columns(self):
         """
         This needs to be implemented in a subclass
-        @returns: columns
-        @rtype: list of L{kiwi.ui.objectlist.Column}
+        :returns: columns
+        :rtype: list of :class:`kiwi.ui.objectlist.Column`
         """
         raise NotImplementedError
