@@ -38,6 +38,8 @@ import gobject
 import gtk
 from gtk import gdk
 
+from kiwi import ValueUnset
+from kiwi.datatypes import ValidationError
 from kiwi.environ import environ
 from kiwi.interfaces import IValidatableProxyWidget
 from kiwi.log import Logger
@@ -48,7 +50,9 @@ from kiwi.ui.proxy import Proxy
 
 log = Logger('kiwi.view')
 
-_non_interactive = (
+validation_log = Logger('kiwi.validation')
+
+_non_interactive = [
     gtk.Label,
     gtk.Alignment,
     gtk.AccelLabel,
@@ -61,7 +65,6 @@ _non_interactive = (
     gtk.HPaned,
     gtk.HSeparator,
     gtk.Layout,
-    gtk.Progress,
     gtk.ProgressBar,
     gtk.ScrolledWindow,
     gtk.Table,
@@ -70,7 +73,10 @@ _non_interactive = (
     gtk.VPaned,
     gtk.VSeparator,
     gtk.Window,
-)
+]
+
+if hasattr(gtk, 'Progress'):
+    _non_interactive.append(gtk.Progress)
 
 color_red = gdk.color_parse('red')
 color_black = gdk.color_parse('black')
@@ -279,7 +285,7 @@ class SlaveView(gobject.GObject):
                             "called toplevel that specifies the "
                             "toplevel widget in it")
 
-        if isinstance(toplevel, gtk.Window):
+        if isinstance(toplevel, (gtk.Window, gtk.Dialog)):
             if toplevel.get_visible():
                 log.warn("Toplevel widget %s (%s) is visible; that's probably "
                          "wrong" % (toplevel, toplevel.get_name()))
@@ -304,7 +310,7 @@ class SlaveView(gobject.GObject):
         # window, so we pull our slave out from it, grab its groups and
         # muerder it later
         shell = glade_adaptor.get_widget(container_name)
-        if not isinstance(shell, gtk.Window):
+        if not isinstance(shell, (gtk.Window, gtk.Dialog)):
             raise TypeError("Container %s should be a Window, found %s" % (
                 container_name, type(shell)))
 
@@ -385,7 +391,7 @@ class SlaveView(gobject.GObject):
         argument is supplied and is a valid view, this view is set as a
         transient for the parent view
 
-        @param parent:
+        :param parent:
         """
 
         self.show()
@@ -452,7 +458,8 @@ class SlaveView(gobject.GObject):
                 # window, it's safe to realize it. If this check isn't
                 # performed, we get a crash as per
                 # http://bugzilla.gnome.org/show_bug.cgi?id=107872
-                if isinstance(widget.get_toplevel(), gtk.Window):
+                if isinstance(widget.get_toplevel(), (gtk.Window,
+                                                      gtk.Dialog)):
                     widget.realize()
                 else:
                     log.warn("get_topmost_widget: widget %s was not realized"
@@ -464,7 +471,8 @@ class SlaveView(gobject.GObject):
                     widget = widget.entry
                 if not widget.flags() & gtk.CAN_FOCUS or \
                     isinstance(widget, (gtk.Label, gtk.HSeparator,
-                                        gtk.VSeparator, gtk.Window)):
+                                        gtk.VSeparator, gtk.Window,
+                                        gtk.Dialog)):
                     continue
 
             if top_widget:
@@ -535,7 +543,7 @@ class SlaveView(gobject.GObject):
 
         shell = slave.get_toplevel()
 
-        if isinstance(shell, gtk.Window): # view with toplevel window
+        if isinstance(shell, (gtk.Window, gtk.Dialog)): # view with toplevel window
             new_widget = shell.get_child()
             shell.remove(new_widget) # remove from window to allow reparent
         else: # slaveview
@@ -555,7 +563,7 @@ class SlaveView(gobject.GObject):
             # accel groups; otherwise complain if we're dropping the
             # accelerators
             win = parent.get_toplevel()
-            if isinstance(win, gtk.Window):
+            if isinstance(win, (gtk.Window, gtk.Dialog)):
                 # use idle_add to be sure we attach the groups as late
                 # as possible and avoid reattaching groups -- see
                 # comment in _attach_groups.
@@ -759,6 +767,20 @@ class SlaveView(gobject.GObject):
                 raise AssertionError("%r does not have a validation-changed "
                                      "signal." % widget)
 
+            if (widget.props.mandatory and
+                widget.props.sensitive and
+                widget.props.visible):
+                try:
+                    value = widget.read()
+                    empty = value is ValueUnset or value is None
+                    validation_log.info("%s: %s=%r (initial)" % (
+                        self.__class__.__name__, widget_name, value))
+                    if empty:
+                        self._validation[widget_name] = False
+                # FIXME: This is probably wrong, but we get improperly placed
+                #        thousand operators in other places that has to be fixed first
+                except ValidationError:
+                    pass
         proxy = Proxy(self, model, widgets)
         self._proxies.append(proxy)
         return proxy
@@ -777,6 +799,8 @@ class SlaveView(gobject.GObject):
                 not child.get_property('sensitive')):
                 value = True
 
+        validation_log.info("%s: %s=%r" % (self.__class__.__name__,
+                                           name, value))
         self._validation[name] = value
 
         self.check_and_notify_validity()
@@ -806,10 +830,10 @@ class SlaveView(gobject.GObject):
     def check_and_notify_validity(self, force=False):
         # Current view is only valid if we have no invalid children
         # their status are stored as values in the dictionary
-        is_valid = True
-        if False in self._validation.values():
-            is_valid = False
+        is_valid = all(self._validation.values())
 
+        validation_log.info("%s: validate state=%r" % (
+            self.__class__.__name__, self._validation))
         # Check if validation really changed
         if self._valid == is_valid and force == False:
             return
@@ -848,7 +872,7 @@ class BaseView(SlaveView):
         SlaveView.__init__(self, toplevel, widgets, gladefile, toplevel_name,
                            domain)
 
-        if not isinstance(self.toplevel, gtk.Window):
+        if not isinstance(self.toplevel, (gtk.Window, gtk.Dialog)):
             raise TypeError("toplevel widget must be a Window "
                             "(or inherit from it),\nfound `%s' %s"
                             % (toplevel, self.toplevel))
@@ -892,7 +916,7 @@ class BaseView(SlaveView):
         # top view is difficult. We used to print a warning here, I
         # removed it for convenience; we might want to put it back when
         # http://bugs.async.com.br/show_bug.cgi?id=682 is fixed
-        elif isinstance(view, gtk.Window):
+        elif isinstance(view, (gtk.Window, gtk.Dialog)):
             self.toplevel.set_transient_for(view)
         else:
             raise TypeError("Parameter to set_transient_for should "
@@ -922,7 +946,7 @@ class BaseView(SlaveView):
         # Check if any of the widgets is interactive
         for v in values:
             if (isinstance(v, gtk.Widget) and not
-                isinstance(v, _non_interactive)):
+                isinstance(v, tuple(_non_interactive))):
                 interactive = v
         if interactive:
             log.warn("No widget is focused in view %s but you have an "
@@ -962,12 +986,11 @@ class BaseView(SlaveView):
         self.toplevel.hide()
         self.quit_if_last()
 
-def _get_gazpacho():
-    try:
-        from kiwi.ui.gazpacholoader import GazpachoWidgetTree
-    except ImportError:
-        return
-    return GazpachoWidgetTree
+_glade_loader_func = None
+
+def set_glade_loader_func(func):
+    global _glade_loader_func
+    _glade_loader_func = func
 
 def _get_libglade():
     try:
@@ -997,6 +1020,9 @@ def _open_glade(view, gladefile, domain):
         raise TypeError(
               "gladefile should be a string, found %s" % type(gladefile))
 
+    if _glade_loader_func:
+        return _glade_loader_func(view, gladefile, domain)
+
     if gladefile.endswith('.ui'):
         directory = os.path.dirname(namedAny(view.__module__).__file__)
         gladefile = os.path.join(directory, gladefile)
@@ -1023,10 +1049,6 @@ def _open_glade(view, gladefile, domain):
     elif 'gaxml-0.1.dtd' in sniff:
         WidgetTree = _get_gaxml()
         loader_name = 'gaxml'
-    # gazpacho: <!DOCTYPE glade-interface SYSTEM "http://gazpacho.sicem.biz/gazpacho-0.1.dtd">
-    elif 'gazpacho-0.1.dtd' in sniff:
-        WidgetTree = _get_gazpacho()
-        loader_name = 'gazpacho.loader'
     else:
         log.warning("Could not determine type/dtd of gladefile %s" % gladefile)
         # Defaulting to builder
